@@ -1,5 +1,4 @@
 import json
-import logging
 import os
 import shutil
 import socket
@@ -7,18 +6,21 @@ import subprocess
 import sys
 import traceback
 import warnings
+from datetime import datetime, timedelta
 from itertools import chain
+from logging import getLogger
 from pathlib import Path
-from sys import stdout, stderr
+from sys import stdout
 from time import sleep
 from typing import Optional, Iterable
 
 import ipynbname
-from chrisdict import AttrDict
+import pandas as pd
 from tabulate import tabulate
 
-from .time import *
-from .util import *
+from chrisbase.time import from_timestamp, now, str_delta
+from chrisbase.util import to_dataframe, tupled, SP, NO, OX
+from chrisdict import AttrDict
 
 sys_stdout = sys.stdout
 sys_stderr = sys.stderr
@@ -129,7 +131,7 @@ class MuteStd:
 
     def __enter__(self):
         try:
-            self.mute_logger = [logging.getLogger(x) for x in self.mute_logger] if self.mute_logger else None
+            self.mute_logger = [getLogger(x) for x in self.mute_logger] if self.mute_logger else None
             if self.mute_logger:
                 for x in self.mute_logger:
                     x.disabled = True
@@ -164,7 +166,7 @@ class MuteStd:
 
 class MyTimer:
     def __init__(self, name=None, prefix=None, postfix=None, verbose=False, mt=0, mb=0, pt=0, pb=0, rt=0, rb=0, rc='-',
-                 file=stdout, line=stderr, flush_sec=0.25, mute_logger=None, mute_warning=None):
+                 file=stdout, flush_sec=None, mute_logger=None, mute_warning=None):
         self.mute = open(os.devnull, 'w')
         self.name = name
         self.prefix = prefix if prefix and len(prefix) > 0 else None
@@ -178,7 +180,6 @@ class MyTimer:
         self.rb: int = rb
         self.rc: str = rc
         self.file = self.mute if not verbose else file or self.mute
-        self.line = self.mute if not verbose else line or self.mute
         self.preout = sys.stdout
         self.preerr = sys.stderr
         assert isinstance(mute_logger, (type(None), str, list, tuple, set))
@@ -192,7 +193,7 @@ class MyTimer:
 
     def __enter__(self):
         try:
-            self.mute_logger = [logging.getLogger(x) for x in self.mute_logger] if self.mute_logger else None
+            self.mute_logger = [getLogger(x) for x in self.mute_logger] if self.mute_logger else None
             if self.mute_logger:
                 for x in self.mute_logger:
                     x.disabled = True
@@ -206,18 +207,18 @@ class MyTimer:
             if self.verbose:
                 if self.mt > 0:
                     for _ in range(self.mt):
-                        print(file=self.line)
+                        print(file=self.file)
                 if self.rt > 0:
                     for _ in range(self.rt):
-                        file_hr(c=self.rc, file=self.line)
+                        file_hr(c=self.rc, file=self.file)
                 if self.name:
-                    print(f'{now()} {self.prefix + SP if self.prefix else NO}[INIT] {self.name}{SP + self.postfix if self.postfix else NO}', file=self.line)
+                    print(f'{now()} {self.prefix + SP if self.prefix else NO}[INIT] {self.name}{SP + self.postfix if self.postfix else NO}', file=self.file)
                     if self.rt > 0:
                         for _ in range(self.rt):
-                            file_hr(c=self.rc, file=self.line)
+                            file_hr(c=self.rc, file=self.file)
                 if self.pt > 0:
                     for _ in range(self.pt):
-                        print(file=self.line)
+                        print(file=self.file)
                 flush_or(sys.stdout, sys.stderr, sec=self.flush_sec if self.flush_sec else None)
             self.t1 = datetime.now()
             return self
@@ -233,18 +234,18 @@ class MyTimer:
             if self.verbose:
                 if self.pb > 0:
                     for _ in range(self.pb):
-                        print(file=self.line)
+                        print(file=self.file)
                 if self.rb > 0:
                     for _ in range(self.rb):
-                        file_hr(c=self.rc, file=self.line)
+                        file_hr(c=self.rc, file=self.file)
                 if self.name:
-                    print(f'{now()} {self.prefix + SP if self.prefix else NO}[EXIT] {self.name}{SP + self.postfix if self.postfix else NO} ($={str_delta(self.td)})', file=self.line)
+                    print(f'{now()} {self.prefix + SP if self.prefix else NO}[EXIT] {self.name}{SP + self.postfix if self.postfix else NO} ($={str_delta(self.td)})', file=self.file)
                     if self.rb > 0:
                         for _ in range(self.rb):
-                            file_hr(c=self.rc, file=self.line)
+                            file_hr(c=self.rc, file=self.file)
                 if self.mb > 0:
                     for _ in range(self.mb):
-                        print(file=self.line)
+                        print(file=self.file)
                 flush_or(sys.stdout, sys.stderr, sec=self.flush_sec if self.flush_sec else None)
             if self.mute_logger:
                 for x in self.mute_logger:
@@ -417,6 +418,15 @@ def remove_dir(path, ignore_errors=False) -> Path:
     return path
 
 
+def remove_dir_check(path, ignore_errors=True, file=None):
+    path = Path(path)
+    print(f"- {str(path):<40}: {OX(path.exists())}", end='', file=file)
+    if path.exists():
+        shutil.rmtree(path, ignore_errors=ignore_errors)
+    print(f" -> {OX(path.exists())}", file=file)
+    return not path.exists()
+
+
 def remove_any(path, sleep_sec=0.0) -> Path:
     path = Path(path)
     if path.exists():
@@ -528,17 +538,6 @@ def load_attrs_with_base(file, base_name='base_conf', pre=None, post=None) -> At
     return AttrDict(merge_dicts(attrs, post))
 
 
-# def normalize_batch_size(targets, attrs, default=1) -> AttrDict:
-#     max_batch_size = attrs.pop('max_batch_size') if 'max_batch_size' in attrs else None
-#     for target in targets:
-#         if attrs[f'{target}_batch_size'] and attrs[f'{target}_batch_size'] < 1:
-#             if f'max_sequence_length={attrs.max_sequence_length}' in max_batch_size and f'precision={attrs.precision}' in max_batch_size[f'max_sequence_length={attrs.max_sequence_length}']:
-#                 attrs[f'{target}_batch_size'] = max_batch_size[f'max_sequence_length={attrs.max_sequence_length}'][f'precision={attrs.precision}']
-#             else:
-#                 attrs[f'{target}_batch_size'] = max(1, default)
-#     return attrs
-
-
 def save_attrs(obj: dict, file, keys=None, excl=None):
     if keys is not None and isinstance(keys, (list, tuple, set)):
         keys = [x for x in keys if x in obj.keys()]
@@ -572,6 +571,34 @@ def run_command(*args, title=None, mt=0, mb=0, pt=0, pb=0, rt=0, rb=0, rc='-', b
             subprocess.run(list(map(str, args)), stdout=None if verbose else scope.mute, stderr=None if verbose else scope.mute)
 
 
+def get_valid_lines(lines,
+                    accumulating_querys=(
+                            ("(Epoch ", "training #1"),
+                            ("(Epoch ", "metering #1"),
+                    )):
+    last_lines = {query: None for query in accumulating_querys}
+    for line in lines:
+        changed = True
+        if len(str(line).strip()) > 0:
+            for query in last_lines:
+                if all(q in line for q in query):
+                    last_lines[query] = line
+                    changed = False
+        if changed:
+            for query in last_lines:
+                if last_lines[query]:
+                    yield last_lines[query]
+                    last_lines[query] = None
+            if len(str(line).strip()) > 0:
+                yield line
+
+
+def trim_output(infile, outfile):
+    infile = Path(infile)
+    outfile = Path(outfile)
+    outfile.write_text('\n'.join(get_valid_lines(all_lines(infile))))
+
+
 def get_hostname():
     return socket.gethostname()
 
@@ -584,7 +611,8 @@ def get_python_path():
     return subprocess.run("which python".split(), stdout=subprocess.PIPE).stdout.decode('utf-8').strip()
 
 
-def set_cuda_path(cuda_paths=("/usr/local/cuda-11", "/usr/local/cuda-11.4", "/usr/local/cuda-11.3", "/usr/local/cuda-11.1")):
+def set_cuda_path(cuda_paths=("/usr/local/cuda-12.0", "/usr/local/cuda-11.4", "/usr/local/cuda-11.3", "/usr/local/cuda-11.1",
+                              "/usr/local/cuda-12", "/usr/local/cuda-11", "/usr/local/cuda")):
     cuda_dir = None
     for cuda_path in cuda_paths:
         if exists_or(cuda_path):
